from werkzeug.exceptions import BadRequest, NotFound
from sqlalchemy.exc import IntegrityError

from flaschengeist import logger
from flaschengeist.database import db
from .models import Drink, DrinkPrice, Ingredient, Tag, DrinkType, DrinkPriceVolume, DrinkIngredient, ExtraIngredient

from math import ceil


def update():
    db.session.commit()


def get_tags():
    return Tag.query.all()


def get_tag(identifier):
    if isinstance(identifier, int):
        retVal = Tag.query.get(identifier)
    elif isinstance(identifier, str):
        retVal = Tag.query.filter(Tag.name == identifier).one_or_none()
    else:
        logger.debug("Invalid identifier type for Tag")
        raise BadRequest
    if not retVal:
        raise NotFound
    return retVal


def create_tag(name):
    try:
        tag = Tag(name=name)
        db.session.add(tag)
        update()
        return tag
    except IntegrityError:
        raise BadRequest("Name already exists")


def rename_tag(identifier, new_name):
    tag = get_tag(identifier)
    tag.name = new_name
    try:
        update()
    except IntegrityError:
        raise BadRequest("Name already exists")


def delete_tag(identifier):
    tag = get_tag(identifier)
    db.session.delete(tag)
    try:
        update()
    except IntegrityError:
        raise BadRequest("Tag still in use")


def get_drink_types():
    return DrinkType.query.all()


def get_drink_type(identifier):
    if isinstance(identifier, int):
        retVal = DrinkType.query.get(identifier)
    elif isinstance(identifier, str):
        retVal = DrinkType.query.filter(Tag.name == identifier).one_or_none()
    else:
        logger.debug("Invalid identifier type for DrinkType")
        raise BadRequest
    if not retVal:
        raise NotFound
    return retVal


def create_drink_type(name):
    try:
        drinkType = DrinkType(name=name)
        db.session.add(drinkType)
        update()
        return drinkType
    except IntegrityError:
        raise BadRequest("Name already exists")


def rename_drink_type(identifier, new_name):
    drink_type = get_drink_type(identifier)
    drink_type.name = new_name
    try:
        update()
    except IntegrityError:
        raise BadRequest("Name already exists")
    return drink_type


def delete_drink_type(identifier):
    drinkType = get_drink_type(identifier)
    db.session.delete(drinkType)
    try:
        update()
    except IntegrityError:
        raise BadRequest("DrinkType still in use")


def get_drinks(name=None):
    if name:
        return Drink.query.filter(Drink.name.contains(name)).all()
    return Drink.query.all()


def get_drink(identifier):
    if isinstance(identifier, int):
        return Drink.query.get(identifier)
    elif isinstance(identifier, str):
        return Drink.query.filter(Tag.name == identifier).one_or_none()
    else:
        logger.debug("Invalid identifier type for Drink")
        raise BadRequest
    raise NotFound


def get_volume(identifier):
    return DrinkPriceVolume.query.get(identifier)


def get_price(identifier):
    if isinstance(identifier, int):
        return DrinkPrice.query.get(identifier)
    raise NotFound


def get_prices(volume_id=None):
    if volume_id:
        return DrinkPrice.query.filter(DrinkPrice.volume_id_ == volume_id).all()
    return DrinkPrice.query.all()


def set_price(identifier, data):
    allowed_keys = DrinkPrice().serialize().keys()
    values = {key: value for key, value in data.items() if key in allowed_keys}
    price = DrinkPrice(**values)
    volume = get_volume(identifier)
    if not volume:
        raise BadRequest
    volume.prices.append(price)
    db.session.add(price)
    db.session.commit()
    return price


def update_price(identifier, data):
    allowed_keys = DrinkPrice().serialize().keys()
    if "id" in data:
        data.pop("id")
    values = {key: value for key, value in data.items() if key in allowed_keys}
    price = get_price(identifier)
    for key, value in values:
        setattr(price, key, value)
    db.session.commit()
    return price


def delete_price(identifier):
    price = get_price(identifier)
    db.session.delete(price)
    db.session.commit()
